@@ -35,10 +35,11 @@
 
 ''' Module providing a transformation script that converts the Schedule of
     the first Invoke to use OpenCL. In order to do this, those kernels that
-    access global data are transformed so as to pass that data by argument. '''
+    access imported data are transformed so as to pass that data by argument.
+'''
 
-from psyclone.transformations import OCLTrans, KernelGlobalsToArguments
-from psyclone.domain.gocean.transformations import \
+from psyclone.transformations import KernelImportsToArguments
+from psyclone.domain.gocean.transformations import GOOpenCLTrans, \
     GOMoveIterationBoundariesInsideKernelTrans
 
 
@@ -58,15 +59,11 @@ def trans(psy):
     invoke = psy.invokes.invoke_list[0]
     sched = invoke.schedule
 
-    # Convert any kernel accesses to global data into arguments
-    ktrans = KernelGlobalsToArguments()
+    # Convert any kernel accesses to imported data into arguments
+    ktrans = KernelImportsToArguments()
     for kern in sched.kernels():
         ktrans.apply(kern)
 
-    # Transform the Schedule
-    cltrans = OCLTrans()
-    cltrans.apply(sched, options={"end_barrier": True})
-
     # Provide kernel-specific OpenCL optimization options
     move_boundaries_trans = GOMoveIterationBoundariesInsideKernelTrans()
     for kern in sched.kernels():
@@ -76,4 +73,8 @@ def trans(psy):
         # Specify the OpenCL queue and workgroup size of the kernel
         kern.set_opencl_options({"queue_number": 1, 'local_size': 4})
 
+    # Transform the Schedule
+    cltrans = GOOpenCLTrans()
+    cltrans.apply(sched, options={"end_barrier": True})
+
     return psy
