@@ -39,15 +39,23 @@
 
 from psyclone.domain.gocean.transformations.gocean_extract_trans \
     import GOceanExtractTrans
+from psyclone.domain.gocean.transformations.gocean_opencl_trans \
+    import GOOpenCLTrans
 from psyclone.domain.gocean.transformations. \
     gocean_move_iteration_boundaries_inside_kernel_trans import \
     GOMoveIterationBoundariesInsideKernelTrans
 from psyclone.domain.gocean.transformations.gocean_loop_fuse_trans \
     import GOceanLoopFuseTrans
+from psyclone.domain.gocean.transformations.gocean_const_loop_bounds_trans \
+    import GOConstLoopBoundsTrans
 
 # The entities in the __all__ list are made available to import directly from
 # this package e.g.:
 # from psyclone.domain.gocean.transformations import GOceanExtractTrans
 
-__all__ = ['GOceanExtractTrans', 'GOMoveIterationBoundariesInsideKernelTrans',
+__all__ = ['GOceanExtractTrans',
+           'GOMoveIterationBoundariesInsideKernelTrans',
+           'GOceanLoopFuseTrans',
+           'GOOpenCLTrans',
+           'GOConstLoopBoundsTrans',
            'GOceanLoopFuseTrans']
