@@ -117,10 +117,6 @@ class GOMoveIterationBoundariesInsideKernelTrans(Transformation):
         :param options: a dictionary with options for transformations.
         :type options: dict of string:values or None
 
-        :returns: 2-tuple of new schedule and memento of transform.
-        :rtype: (:py:class:`psyclone.gocean1p0.GOInvokeSchedule`, \
-                 :py:class:`psyclone.undoredo.Memento`)
-
         '''
         self.validate(node, options)
 
@@ -131,16 +127,16 @@ class GOMoveIterationBoundariesInsideKernelTrans(Transformation):
         cursor = outer_loop.position
 
         # Make sure the boundary symbols in the PSylayer exist
-        inv_xstart = invoke_st.symbol_from_tag(
+        inv_xstart = invoke_st.find_or_create_tag(
             "xstart_" + node.name, root_name="xstart", symbol_type=DataSymbol,
             datatype=INTEGER_TYPE)
-        inv_xstop = invoke_st.symbol_from_tag(
+        inv_xstop = invoke_st.find_or_create_tag(
             "xstop_" + node.name, root_name="xstop", symbol_type=DataSymbol,
             datatype=INTEGER_TYPE)
-        inv_ystart = invoke_st.symbol_from_tag(
+        inv_ystart = invoke_st.find_or_create_tag(
             "ystart_" + node.name, root_name="ystart", symbol_type=DataSymbol,
             datatype=INTEGER_TYPE)
-        inv_ystop = invoke_st.symbol_from_tag(
+        inv_ystop = invoke_st.find_or_create_tag(
             "ystop_" + node.name, root_name="ystop", symbol_type=DataSymbol,
             datatype=INTEGER_TYPE)
 
@@ -241,8 +237,6 @@ class GOMoveIterationBoundariesInsideKernelTrans(Transformation):
         if_statement = IfBlock.create(condition, [Return()])
         kschedule.children.insert(0, if_statement)
 
-        return node.root, None
-
 
 # For Sphinx AutoAPI documentation generation
 __all__ = ['GOMoveIterationBoundariesInsideKernelTrans']
