@@ -69,7 +69,7 @@ class CreateNemoKernelTrans(Transformation):
     >>> print(psyir.view(colour=False, indent="  "))
     FileContainer[]
       Routine[name:'sub']
-        0: Loop[type='None', field_space='None', it_space='None']
+        0: Loop[variable='ji']
           Literal[value:'1', Scalar<INTEGER, UNDEFINED>]
           Literal[value:'10', Scalar<INTEGER, UNDEFINED>]
           Literal[value:'1', Scalar<INTEGER, UNDEFINED>]
