@@ -31,8 +31,8 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 # -----------------------------------------------------------------------------
-# Author: R. W. Ford, STFC Daresbury Lab.
-# Modified by: A. R. Porter, STFC Daresbury Lab.
+# Author: R. W. Ford, STFC Daresbury Lab
+# Modified by: A. R. Porter and N. Nobre, STFC Daresbury Lab
 
 '''SIR PSyIR backend. Generates SIR code from PSyIR nodes. Currently
 limited to PSyIR Kernel schedules as PSy-layer PSyIR already has a
@@ -41,8 +41,6 @@ gen() method to generate Fortran.
 '''
 from __future__ import absolute_import
 
-import six
-
 from psyclone.nemo import NemoLoop, NemoKern
 from psyclone.psyir.backend.visitor import PSyIRVisitor, VisitorError
 from psyclone.psyir.nodes import ArrayReference, BinaryOperation, Literal, \
@@ -79,8 +77,8 @@ def gen_stencil(node):
     '''
     if not isinstance(node, ArrayReference):
         raise VisitorError(
-            "gen_stencil expected an ArrayReference as input but found '{0}'."
-            "".format(type(node)))
+            f"gen_stencil expected an ArrayReference as input but found "
+            f"'{type(node)}'.")
     dims = []
     for child in node.children:
         if isinstance(child, Reference):
@@ -94,18 +92,15 @@ def gen_stencil(node):
                     dims.append(child.children[1].value)
                 else:
                     raise VisitorError(
-                        "gen_stencil unsupported stencil operator found "
-                        "'{0}'. Expecting '+' or '-'."
-                        "".format(child.operator.name))
+                        f"gen_stencil unsupported stencil operator found "
+                        f"'{child.operator.name}'. Expecting '+' or '-'.")
             else:
                 raise VisitorError(
-                    "gen_stencil unsupported stencil index found '{0}'."
-                    "".format(str(child)))
+                    f"gen_stencil unsupported stencil index found '{child}'.")
         else:
             raise VisitorError(
-                "gen_stencil unsupported (non-stencil) index found '{0}'."
-                "".format(str(child)))
-    return "[{0}]".format(", ".join(dims))
+               f"gen_stencil unsupported (non-stencil) index found '{child}'.")
+    return f"[{', '.join(dims)}]"
 
 
 class SIRWriter(PSyIRVisitor):
@@ -159,15 +154,14 @@ class SIRWriter(PSyIRVisitor):
         '''
         if not self._skip_nodes:
             raise VisitorError(
-                "Class SIRWriter method node_node(), unsupported node "
-                "found '{0}'".format(type(node)))
-        result = "{0}[ {1} start ]\n".format(self._nindent,
-                                             type(node).__name__)
+                f"Class SIRWriter method node_node(), unsupported node "
+                f"found '{type(node)}'")
+        result = f"{self._nindent}[ {type(node).__name__} start ]\n"
         self._depth += 1
         for child in node.children:
             result += self._visit(child)
         self._depth -= 1
-        result += "{0}[ {1} end ]\n".format(self._nindent, type(node).__name__)
+        result += f"{self._nindent}[ {type(node).__name__} end ]\n"
         return result
 
     def nemoloop_node(self, loop_node):
@@ -207,23 +201,22 @@ class SIRWriter(PSyIRVisitor):
                 "Child of child of child of loop should be a NemoKern.")
 
         # The interval values are hardcoded for the moment (see #470).
-        result = ("{0}interval = make_interval(Interval.Start, Interval.End, "
-                  "0, 0)\n".format(self._nindent))
-        result += ("{0}body_ast = make_ast([\n".format(self._nindent))
+        result = f"{self._nindent}interval = "\
+                 f"make_interval(Interval.Start, Interval.End, 0, 0)\n"
+        result += f"{self._nindent}body_ast = make_ast([\n"
         self._depth += 1
         result += self.nemokern_node(loop_content[0])
         self._depth -= 1
         # Remove the trailing comma if there is one as this is the
         # last entry in make_ast.
         result = result.rstrip(",\n") + "\n"
-        result += "{0}])\n".format(self._nindent)
+        result += f"{self._nindent}])\n"
         # For the moment there is a hard coded assumption that the
         # vertical looping is in the forward (1..n) direction (see
         # #470).
-        result += (
-            "{0}vertical_region_fns.append(make_vertical_region_decl_stmt("
-            "body_ast, interval, VerticalRegion.Forward))\n"
-            "".format(self._nindent))
+        result += f"{self._nindent}vertical_region_fns.append("\
+                  f"make_vertical_region_decl_stmt(body_ast, interval, "\
+                  f"VerticalRegion.Forward))\n"
         return result
 
     def nemokern_node(self, node):
@@ -262,33 +255,30 @@ class SIRWriter(PSyIRVisitor):
         exec_statements = ""
         for child in node.children:
             exec_statements += self._visit(child)
-        result += "{0}\n".format(exec_statements)
+        result += f"{exec_statements}\n"
         # The file name is hard coded at the moment.
-        result += (
-            "{0}hir = make_sir(stencil_name+\".cpp\", "
-            "AST.GridType.Value(\"Cartesian\"), [\n"
-            "{0}{1}make_stencil(\n"
-            "{0}{1}{1}stencil_name,\n"
-            "{0}{1}{1}make_ast(vertical_region_fns),\n"
-            "{0}{1}{1}[".format(self._nindent, self._indent))
+        result += (self._nindent + "hir = make_sir(stencil_name+\".cpp\", "
+                   "AST.GridType.Value(\"Cartesian\"), [\n"
+                   + self._nindent + self._indent + "make_stencil(\n"
+                   + self._nindent + self._indent * 2 + "stencil_name,\n"
+                   + self._nindent + self._indent * 2
+                   + "make_ast(vertical_region_fns),\n"
+                   + self._nindent + self._indent * 2 + "[")
         functions = []
         for name in self._field_names:
             functions.append(
-                "make_field(\"{0}\", make_field_dimensions_cartesian())"
-                "".format(name))
+                f"make_field(\"{name}\", make_field_dimensions_cartesian())")
         # The current assumption is that scalars are temporaries. This
         # is not necessarily correct and this problem is captured in
         # issue #521. Scalar temporaries can be declared as field
         # temporaries as the Dawn backend works out what is required.
         for name in self._scalar_names:
             functions.append(
-                "make_field(\"{0}\", make_field_dimensions_cartesian(), "
-                "is_temporary=True)".format(name))
+                f"make_field(\"{name}\", make_field_dimensions_cartesian(), "
+                f"is_temporary=True)")
         result += ", ".join(functions)
         result += "]\n"
-        result += (
-            "{0}{1})\n"
-            "{0}])\n".format(self._nindent, self._indent))
+        result += f"{self._nindent}{self._indent})\n{self._nindent}])\n"
         return result
 
     def assignment_node(self, node):
@@ -306,12 +296,11 @@ class SIRWriter(PSyIRVisitor):
         lhs = self._visit(node.lhs)
         rhs = self._visit(node.rhs)
         self._depth -= 1
-        result = ("{0}make_assignment_stmt(\n{1},\n{2}"
-                  "".format(self._nindent, lhs, rhs))
+        result = f"{self._nindent}make_assignment_stmt(\n{lhs},\n{rhs}"
         # For better formatting, remove the newline if one exists.
         result = result.rstrip("\n")
         result += ",\n"
-        result += "{0}{1}\"=\"),\n".format(self._nindent, self._indent)
+        result += f"{self._nindent}{self._indent}\"=\"),\n"
         return result
 
     def binaryoperation_node(self, node):
@@ -348,18 +337,16 @@ class SIRWriter(PSyIRVisitor):
         try:
             oper = binary_operators[node.operator]
         except KeyError as err:
-            six.raise_from(VisitorError(
-                "Method binaryoperation_node in class SIRWriter, unsupported "
-                "operator '{0}' found.".format(str(node.operator))),
-                err)
+            raise VisitorError(
+                f"Method binaryoperation_node in class SIRWriter, unsupported "
+                f"operator '{node.operator}' found.") from err
         rhs = self._visit(node.children[1])
         self._depth -= 1
-        result = "{0}make_binary_operator(\n{1}".format(self._nindent, lhs)
+        result = f"{self._nindent}make_binary_operator(\n{lhs}"
         # For better formatting, remove the newline if one exists.
-        result = result.rstrip("\n")
-        result += ",\n"
-        result += ("{0}{1}\"{2}\",\n{3}\n{0}{1})\n"
-                   "".format(self._nindent, self._indent, oper, rhs))
+        result = result.rstrip("\n") + ",\n"
+        result += f"{self._nindent}{self._indent}\"{oper}\",\n{rhs}\n"\
+                  f"{self._nindent}{self._indent})\n"
         return result
 
     def reference_node(self, node):
@@ -387,8 +374,7 @@ class SIRWriter(PSyIRVisitor):
         # is required).
         self._scalar_names.add(node.name)
 
-        return "{0}make_field_access_expr(\"{1}\")".format(self._nindent,
-                                                           node.name)
+        return f"{self._nindent}make_field_access_expr(\"{node.name}\")"
 
     def arrayreference_node(self, node):
         '''This method is called when an ArrayReference instance is found in
@@ -402,8 +388,8 @@ class SIRWriter(PSyIRVisitor):
 
         '''
         stencil = gen_stencil(node)
-        result = ("{0}make_field_access_expr(\"{1}\", {2})"
-                  "".format(self._nindent, node.name, stencil))
+        result = f"{self._nindent}make_field_access_expr(\"{node.name}\", "\
+                 f"{stencil})"
         # _field_names is a set so duplicates will be ignored. It
         # captures all unique field names as the SIR declares field
         # names after the computation.
@@ -425,13 +411,12 @@ class SIRWriter(PSyIRVisitor):
         try:
             datatype = TYPE_MAP_TO_SIR[node.datatype.intrinsic]
         except KeyError as err:
-            six.raise_from(VisitorError(
-                "PSyIR type '{0}' has no representation in the SIR backend."
-                "".format(str(node.datatype))),
-                err)
+            raise VisitorError(
+                f"PSyIR type '{node.datatype}' has no representation in the "
+                f"SIR backend.") from err
 
-        return ("{0}make_literal_access_expr(\"{1}\", {2})"
-                "".format(self._nindent, result, datatype))
+        return f"{self._nindent}make_literal_access_expr(\"{result}\", "\
+               f"{datatype})"
 
     def unaryoperation_node(self, node):
         '''This method is called when a UnaryOperation instance is found in
@@ -454,10 +439,9 @@ class SIRWriter(PSyIRVisitor):
         try:
             oper = unary_operators[node.operator]
         except KeyError as err:
-            six.raise_from(VisitorError(
-                "Method unaryoperation_node in class SIRWriter, unsupported "
-                "operator '{0}' found.".format(str(node.operator))),
-                err)
+            raise VisitorError(
+                f"Method unaryoperation_node in class SIRWriter, unsupported "
+                f"operator '{node.operator}' found.") from err
         if isinstance(node.children[0], Literal):
             # The unary minus operator is being applied to a
             # literal. This is a special case as the literal value can
@@ -468,12 +452,12 @@ class SIRWriter(PSyIRVisitor):
                 # The '-' operator can only be applied to REAL and INTEGER
                 # datatypes.
                 raise VisitorError(
-                    "PSyIR type '{0}' does not work with the '-' operator."
-                    "".format(str(literal.datatype)))
+                    f"PSyIR type '{literal.datatype}' does not work with the "
+                    f"'-' operator.")
             result = literal.value
             datatype = TYPE_MAP_TO_SIR[literal.datatype.intrinsic]
-            return ("{0}make_literal_access_expr(\"{1}{2}\", {3})"
-                    "".format(self._nindent, oper, result, datatype))
+            return f"{self._nindent}make_literal_access_expr(\"{oper}{result}"\
+                   f"\", {datatype})"
 
         # The unary minus operator is being applied to something that
         # is not a literal. Default to REAL as we currently have no
@@ -481,13 +465,12 @@ class SIRWriter(PSyIRVisitor):
         # -1.0 * x.
         datatype = TYPE_MAP_TO_SIR[ScalarType.Intrinsic.REAL]
         self._depth += 1
-        lhs = "{0}make_literal_access_expr(\"-1.0\", {1})".format(
-            self._nindent, datatype)
-        operator = "{0}\"*\"".format(self._nindent)
+        lhs = f"{self._nindent}make_literal_access_expr(\"-1.0\", {datatype})"
+        operator = f"{self._nindent}\"*\""
         rhs = self._visit(node.children[0])
         self._depth -= 1
-        result = "{0}make_binary_operator(\n{1},\n{2},\n{3})\n".format(
-            self._nindent, lhs, operator, rhs)
+        result = f"{self._nindent}make_binary_operator"\
+                 f"(\n{lhs},\n{operator},\n{rhs})\n"
         return result
 
     def ifblock_node(self, node):
@@ -502,21 +485,20 @@ class SIRWriter(PSyIRVisitor):
 
         '''
         cond_expr = self._visit(node.condition)
-        cond_part = ("make_expr_stmt({0})"
-                     "".format(cond_expr.lstrip().rstrip(",\n")))
+        cond_part = "make_expr_stmt(" + cond_expr.lstrip().rstrip(",\n") + ")"
 
         then_statements = self._visit(node.if_body).lstrip().rstrip(",\n")
-        then_part = "make_block_stmt([{0}])".format(then_statements)
+        then_part = "make_block_stmt([" + then_statements + "])"
 
         if node.else_body:
             else_statements = self._visit(node.else_body)
-            else_part = ("make_block_stmt([{0}])"
-                         "".format(else_statements.lstrip().rstrip(",\n")))
+            else_part = "make_block_stmt([" + \
+                        else_statements.lstrip().rstrip(",\n") + "])"
         else:
             else_part = "None"
 
-        return ("{0}make_if_stmt({1}, {2}, {3}),\n"
-                "".format(self._nindent, cond_part, then_part, else_part))
+        return f"{self._nindent}make_if_stmt({cond_part}, {then_part}, "\
+               f"{else_part}),\n"
 
     def schedule_node(self, node):
         '''This method is called when a Schedule instance is found in the
