@@ -61,19 +61,15 @@ class Loop(Statement):
     loops. This means the loop is bounded by start/stop/step expressions
     evaluated before the loop starts.)
 
-    :param parent: parent of this node in the PSyIR.
-    :type parent: sub-class of :py:class:`psyclone.psyir.nodes.Node`
     :param variable: optional reference to the loop iterator \
         variable. Defaults to None.
-    :type variable: :py:class:`psyclone.psyir.symbols.DataSymbol` or \
-        `NoneType`
-    :param valid_loop_types: a list of loop types that are specific \
-        to a particular API.
-    :type valid_loop_types: list of str
+    :type variable: Optional[:py:class:`psyclone.psyir.symbols.DataSymbol`]
     :param annotations: One or more labels that provide additional information\
-          about the node (primarily relating to the input code that it was \
-          created from).
-    :type annotations: list of str
+        about the node (primarily relating to the input code that it was \
+        created from).
+    :type annotations: Optional[List[str]]
+    :param kwargs: additional keyword arguments provided to the PSyIR node.
+    :type kwargs: unwrapped dict.
 
     :raises InternalError: if the 'was_single_stmt' annotation is supplied \
                            without the 'was_where' annotation.
@@ -85,10 +81,7 @@ class Loop(Statement):
     _text_name = "Loop"
     _colour = "red"
 
-    def __init__(self, parent=None, variable=None, valid_loop_types=None,
-                 annotations=None):
-        super(Loop, self).__init__(self, parent=parent,
-                                   annotations=annotations)
+    def __init__(self, variable=None, annotations=None, **kwargs):
         # Although the base class checks on the annotations individually, we
         # need to do further checks here
         if annotations:
@@ -99,34 +92,17 @@ class Loop(Statement):
                     f"must also have the 'was_where' annotation but"
                     f" got: {annotations}")
 
-        # we need to determine whether this is a built-in or kernel
-        # call so our schedule can do the right thing.
-
-        if valid_loop_types is None:
-            self._valid_loop_types = []
-        else:
-            self._valid_loop_types = valid_loop_types
-        self._loop_type = None        # inner, outer, colour, colours, ...
-        self._field = None
-        self._field_name = None       # name of the field
-        self._field_space = None      # v0, v1, ...,     cu, cv, ...
-        self._iteration_space = None  # cells, ...,      cu, cv, ...
-        self._kern = None             # Kernel associated with this loop
-
-        # TODO replace iterates_over with iteration_space
-        self._iterates_over = "unknown"
-
-        if variable:
-            # The variable might not be provided when the loop is
-            # first created so only check if it is.
-            self._check_variable(variable)
-        self._variable = variable
+        super().__init__(self, annotations=annotations, **kwargs)
+        # Call the variable setter for error checking
+        self._variable = None
+        if variable is not None:
+            self.variable = variable
 
     def __eq__(self, other):
         '''
         Checks whether two nodes are equal. Two Loop nodes are equal
-        if they have equal loop_type, field, field_name, field_space
-        iteraction_space, kernel and variable.
+        if they have the same iteration variable and their children are
+        equal.
 
         :param object other: the object to check equality to.
 
@@ -134,15 +110,6 @@ class Loop(Statement):
         :rtype: bool
         '''
         is_eq = super().__eq__(other)
-        is_eq = is_eq and self.loop_type == other.loop_type
-        is_eq = is_eq and self.field == other.field
-        is_eq = is_eq and self.field_name == other.field_name
-        is_eq = is_eq and self.field_space == other.field_space
-        is_eq = is_eq and self.iteration_space == other.iteration_space
-        is_eq = is_eq and self.kernel == other.kernel
-        # pylint: disable=protected-access
-        is_eq = is_eq and self._iterates_over == other._iterates_over
-
         is_eq = is_eq and self.variable == other.variable
 
         return is_eq
@@ -186,8 +153,8 @@ class Loop(Statement):
         return (position in (0, 1, 2) and isinstance(child, DataNode)) or (
             position == 3 and isinstance(child, Schedule))
 
-    @staticmethod
-    def create(variable, start, stop, step, children):
+    @classmethod
+    def create(cls, variable, start, stop, step, children):
         '''Create a Loop instance given valid instances of a variable,
         start, stop and step nodes, and a list of child nodes for the
         loop body.
@@ -215,14 +182,14 @@ class Loop(Statement):
             are not of the expected type.
 
         '''
-        Loop._check_variable(variable)
+        cls._check_variable(variable)
 
         if not isinstance(children, list):
             raise GenerationError(
                 f"children argument in create method of Loop class "
                 f"should be a list but found '{type(children).__name__}'.")
 
-        loop = Loop(variable=variable)
+        loop = cls(variable=variable)
         schedule = Schedule(parent=loop, children=children)
         loop.children = [start, stop, step, schedule]
         return loop
@@ -325,42 +292,7 @@ class Loop(Statement):
         '''
         _, position = self._find_position(self.ancestor(Routine))
 
-        if self.loop_type:
-            name = f"loop_[{self.loop_type}]_{position}"
-        else:
-            name = "loop_" + str(position)
-        return name
-
-    @property
-    def valid_loop_types(self):
-        '''
-        :returns: the (domain-specific) loop types allowed by this instance.
-        :rtype: list of str
-        '''
-        return self._valid_loop_types
-
-    @property
-    def loop_type(self):
-        '''
-        :returns: the (domain-specific) type of this loop.
-        :rtype: str
-        '''
-        return self._loop_type
-
-    @loop_type.setter
-    def loop_type(self, value):
-        '''
-        Set the type of this Loop.
-
-        :param str value: the type of this loop.
-        :raises GenerationError: if the specified value is not a recognised \
-                                 loop type.
-        '''
-        if value not in self._valid_loop_types:
-            raise GenerationError(
-                f"Error, loop_type value ({value}) is invalid. Must be one of "
-                f"{self._valid_loop_types}.")
-        self._loop_type = value
+        return "loop_" + str(position)
 
     def node_str(self, colour=True):
         '''
@@ -373,55 +305,7 @@ class Loop(Statement):
         :rtype: str
 
         '''
-        return (f"{self.coloured_name(colour)}[type='{self._loop_type}', "
-                f"field_space='{self._field_space}', "
-                f"it_space='{self.iteration_space}']")
-
-    @property
-    def field_space(self):
-        return self._field_space
-
-    @field_space.setter
-    def field_space(self, my_field_space):
-        self._field_space = my_field_space
-
-    @property
-    def field_name(self):
-        return self._field_name
-
-    @property
-    def field(self):
-        return self._field
-
-    @field_name.setter
-    def field_name(self, my_field_name):
-        self._field_name = my_field_name
-
-    @property
-    def iteration_space(self):
-        return self._iteration_space
-
-    @iteration_space.setter
-    def iteration_space(self, it_space):
-        self._iteration_space = it_space
-
-    @property
-    def kernel(self):
-        '''
-        :returns: the kernel object associated with this Loop (if any).
-        :rtype: :py:class:`psyclone.psyGen.Kern`
-        '''
-        return self._kern
-
-    @kernel.setter
-    def kernel(self, kern):
-        '''
-        Setter for kernel object associated with this loop.
-
-        :param kern: a kernel object.
-        :type kern: :py:class:`psyclone.psyGen.Kern`
-        '''
-        self._kern = kern
+        return f"{self.coloured_name(colour)}[variable='{self.variable.name}']"
 
     @property
     def variable(self):
@@ -449,8 +333,6 @@ class Loop(Statement):
         name = self.__class__.__name__
         result = name + "["
         result += "variable:'" + self.variable.name
-        if self.loop_type:
-            result += "', loop_type:'" + self._loop_type
         result += "']\n"
         for entity in self._children:
             result += str(entity) + "\n"
@@ -488,77 +370,6 @@ class Loop(Statement):
             child.reference_accesses(var_accesses)
             var_accesses.next_location()
 
-    def has_inc_arg(self):
-        ''' Returns True if any of the Kernels called within this
-        loop have an argument with INC access. Returns False otherwise '''
-        for kern_call in self.coded_kernels():
-            for arg in kern_call.arguments.args:
-                if arg.access == AccessType.INC:
-                    return True
-        return False
-
-    def unique_modified_args(self, arg_type):
-        '''Return all unique arguments of the given type from kernels inside
-        this loop that are modified.
-
-        :param str arg_type: the type of kernel argument (e.g. field, \
-                             operator) to search for.
-        :returns: all unique arguments of the given type from kernels inside \
-            this loop that are modified.
-        :rtype: list of :py:class:`psyclone.psyGen.DynKernelArgument`
-        '''
-        arg_names = []
-        args = []
-        for call in self.kernels():
-            for arg in call.arguments.args:
-                if arg.argument_type.lower() == arg_type:
-                    if arg.access != AccessType.READ:
-                        if arg.name not in arg_names:
-                            arg_names.append(arg.name)
-                            args.append(arg)
-        return args
-
-    def unique_fields_with_halo_reads(self):
-        ''' Returns all fields in this loop that require at least some
-        of their halo to be clean to work correctly.
-
-        :returns: fields in this loop that require at least some of their \
-            halo to be clean to work correctly.
-        :rtype: list of :py:class:`psyclone.psyGen.Argument`
-        '''
-
-        unique_fields = []
-        unique_field_names = []
-
-        for call in self.kernels():
-            for arg in call.arguments.args:
-                if self._halo_read_access(arg):
-                    if arg.name not in unique_field_names:
-                        unique_field_names.append(arg.name)
-                        unique_fields.append(arg)
-        return unique_fields
-
-    def args_filter(self, arg_types=None, arg_accesses=None, unique=False):
-        '''Return all arguments of type arg_types and arg_accesses. If these
-        are not set then return all arguments. If unique is set to
-        True then only return uniquely named arguments'''
-        # Avoid circular import
-        # pylint: disable=import-outside-toplevel
-        from psyclone.psyGen import args_filter
-        all_args = []
-        all_arg_names = []
-        for call in self.kernels():
-            call_args = args_filter(call.arguments.args, arg_types,
-                                    arg_accesses)
-            if unique:
-                for arg in call_args:
-                    if arg.name not in all_arg_names:
-                        all_args.append(arg)
-                        all_arg_names.append(arg.name)
-            else:
-                all_args.extend(call_args)
-        return all_args
-
     def gen_code(self, parent):
         '''
         Generate the Fortran Loop and any associated code.
@@ -608,21 +419,3 @@ class Loop(Statement):
         my_decl = DeclGen(parent, datatype="integer",
                           entity_decls=[self.variable.name])
         parent.add(my_decl)
-
-    def _halo_read_access(self, arg):
-        '''Determines whether the supplied argument has (or might have) its
-        halo data read within this loop. Returns True if it does, or if
-        it might and False if it definitely does not.
-
-        :param arg: an argument contained within this loop.
-        :type arg: :py:class:`psyclone.psyGen.KernelArgument`
-
-        :return: True if the argument reads, or might read from the \
-                 halo and False otherwise.
-        :rtype: bool
-
-        :raises NotImplementedError: This is an abstract method.
-
-        '''
-        raise NotImplementedError("This method needs to be implemented by the "
-                                  "APIs that support distributed memory.")
