@@ -1,7 +1,7 @@
 # -----------------------------------------------------------------------------
 # BSD 3-Clause License
 #
-# Copyright (c) 2020-2021, Science and Technology Facilities Council.
+# Copyright (c) 2020-2022, Science and Technology Facilities Council.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -33,14 +33,13 @@
 # -----------------------------------------------------------------------------
 # Author: A. R. Porter, STFC Daresbury Lab
 # Modified by: R. W. Ford, STFC Daresbury Lab
+# Modified by: S. Siso, STFC Daresbury Lab
 # -----------------------------------------------------------------------------
 
 ''' This module contains the Routine node implementation.'''
 
-import six
-
 from psyclone.psyir.nodes.schedule import Schedule
-from psyclone.psyir.symbols import DataSymbol, RoutineSymbol, DeferredType
+from psyclone.psyir.symbols import DataSymbol, RoutineSymbol, NoType
 from psyclone.psyir.nodes.node import Node
 from psyclone.psyir.symbols.symboltable import SymbolTable
 from psyclone.psyir.nodes.commentable_mixin import CommentableMixin
@@ -54,8 +53,8 @@ class Routine(Schedule, CommentableMixin):
     :param str name: the name of this routine.
     :param bool is_program: whether this Routine represents the entry point \
                             into a program (e.g. Fortran Program or C main()).
-    :param parent: the parent node of this Routine node in the PSyIR.
-    :type parent: :py:class:`psyclone.psyir.nodes.Node` or NoneType
+    :param kwargs: additional keyword arguments provided to the super class.
+    :type kwargs: unwrapped dict.
 
     :raises TypeError: if any of the supplied arguments are of the wrong type.
 
@@ -64,22 +63,40 @@ class Routine(Schedule, CommentableMixin):
     _children_valid_format = "[Statement]*"
     _text_name = "Routine"
 
-    def __init__(self, name, is_program=False, parent=None):
-        super(Routine, self).__init__(parent=parent)
+    def __init__(self, name, is_program=False, **kwargs):
+        super().__init__(**kwargs)
 
-        # Name is set-up by the name setter property
-        self._name = None
-        self.name = name
         self._return_symbol = None
+        self._name = None
+        # Name is set-up by the name setter property
+        self.name = name
 
         if not isinstance(is_program, bool):
-            raise TypeError("Routine argument 'is_program' must be a bool but "
-                            "got '{0}'".format(type(is_program).__name__))
+            raise TypeError(f"Routine argument 'is_program' must be a bool "
+                            f"but got '{type(is_program).__name__}'")
         self._is_program = is_program
 
+    def __eq__(self, other):
+        '''
+        Checks whether two nodes are equal. Two Routine nodes are equal
+        if they haev the same name, same return symbol, same is_program and
+        the inherited __eq__ is True.
+
+        :param object other: the object to check equality to.
+
+        :returns: whether other is equal to self.
+        :rtype: bool
+        '''
+        is_eq = super().__eq__(other)
+        is_eq = is_eq and self.name == other.name
+        is_eq = is_eq and self.is_program == other.is_program
+        is_eq = is_eq and self.return_symbol == other.return_symbol
+
+        return is_eq
+
     @classmethod
     def create(cls, name, symbol_table, children, is_program=False,
-               return_symbol=None):
+               return_symbol_name=None):
         '''Create an instance of the supplied class given a name, a symbol
         table and a list of child nodes. This is implemented as a classmethod
         so that it is able to act as a Factory for subclasses - e.g. it
@@ -92,10 +109,9 @@ class Routine(Schedule, CommentableMixin):
         :type children: list of :py:class:`psyclone.psyir.nodes.Node`
         :param bool is_program: whether this Routine represents the entry \
             point into a program (i.e. Fortran Program or C main()).
-        :param return_symbol: the Symbol that holds the return value of this \
-            routine (if any). Must be present in the supplied symbol_table.
-        :type return_symbol: :py:class:`psyclone.psyir.symbols.DataType` or \
-            NoneType
+        :param str return_symbol_name: name of the symbol that holds the \
+            return value of this routine (if any). Must be present in the \
+            supplied symbol table.
 
         :returns: an instance of `cls`.
         :rtype: :py:class:`psyclone.psyGen.Routine` or subclass
@@ -104,37 +120,32 @@ class Routine(Schedule, CommentableMixin):
             are not of the expected type.
 
         '''
-        if not isinstance(name, six.string_types):
+        if not isinstance(name, str):
             raise TypeError(
-                "name argument in create method of Routine class "
-                "should be a string but found '{0}'."
-                "".format(type(name).__name__))
+                f"name argument in create method of Routine class "
+                f"should be a string but found '{type(name).__name__}'.")
         if not isinstance(symbol_table, SymbolTable):
             raise TypeError(
-                "symbol_table argument in create method of Routine "
-                "class should be a SymbolTable but found '{0}'."
-                "".format(type(symbol_table).__name__))
+                f"symbol_table argument in create method of Routine class "
+                f"should be a SymbolTable but found "
+                f"'{type(symbol_table).__name__}'.")
         if not isinstance(children, list):
             raise TypeError(
-                "children argument in create method of Routine class "
-                "should be a list but found '{0}'."
-                "".format(type(children).__name__))
+                f"children argument in create method of Routine class "
+                f"should be a list but found '{type(children).__name__}'.")
         for child in children:
             if not isinstance(child, Node):
                 raise TypeError(
-                    "child of children argument in create method of "
-                    "Routine class should be a PSyIR Node but "
-                    "found '{0}'.".format(type(child).__name__))
+                    f"child of children argument in create method of "
+                    f"Routine class should be a PSyIR Node but "
+                    f"found '{type(child).__name__}'.")
 
-        kern = cls(name)
-        # pylint: disable=protected-access
-        kern._is_program = is_program
-        kern._symbol_table = symbol_table
-        symbol_table._node = kern
-        kern.children = children
-        if return_symbol:
-            kern.return_symbol = return_symbol
-        return kern
+        routine = cls(name, is_program=is_program, symbol_table=symbol_table)
+        routine.children = children
+        if return_symbol_name:
+            routine.return_symbol = routine.symbol_table.lookup(
+                                       return_symbol_name, scope_limit=routine)
+        return routine
 
     def node_str(self, colour=True):
         ''' Returns the name of this node with (optional) control codes
@@ -174,27 +185,42 @@ class Routine(Schedule, CommentableMixin):
         :param str new_name: new name for the Routine.
 
         :raises TypeError: if new_name is not a string.
+        :raises KeyError: if there already is a different named symbol with \
+            the 'own_routine_tag' in the symbol_table.
 
         '''
-        if not isinstance(new_name, six.string_types):
-            raise TypeError("Routine name must be a str but got "
-                            "'{0}'".format(type(new_name).__name__))
+        if not isinstance(new_name, str):
+            raise TypeError(f"Routine name must be a str but got "
+                            f"'{type(new_name).__name__}'")
+        # TODO #1200 The name is duplicated in the _name attribute and the
+        # symbol.name that there is in the local symbol table. This setter
+        # updates both but note that a better solution is needed because
+        # renaming the symbol_table symbol alone would make it inconsistent.
         if not self._name:
+            # If the 'own_routine_symbol' tag already exist check that is
+            # consistent with the given routine name.
+            if 'own_routine_symbol' in self.symbol_table.tags_dict:
+                existing_symbol = self.symbol_table.lookup_with_tag(
+                        'own_routine_symbol', scope_limit=self)
+                if existing_symbol.name.lower() == new_name.lower():
+                    self._name = new_name
+                    return  # The preexisting symbol already matches
+                # Otherwise raise an exception
+                raise KeyError(
+                    f"Can't assign {new_name} as the routine name because "
+                    f"its symbol table contains a symbol ({existing_symbol}) "
+                    f"already tagged as 'own_routine_symbol'.")
+
             self._name = new_name
-            # TODO #1200 naming the routine should not create a symbol and
-            # assign it a type!
-            self.symbol_table.add(
-                RoutineSymbol(new_name, DeferredType()),
-                tag='own_routine_symbol')
+            # Since the constructor can not mark methods as functions directly
+            # the symbol will always start being NoType and must be updated
+            # if a return_value type is provided.
+            self.symbol_table.add(RoutineSymbol(new_name, NoType()),
+                                  tag='own_routine_symbol')
         elif self._name != new_name:
-            old_symbol = self.symbol_table.lookup(self._name)
-            self.symbol_table.remove(old_symbol)
+            symbol = self.symbol_table.lookup(self._name)
             self._name = new_name
-            # TODO #1200 naming the routine should not create a symbol and
-            # assign it a type!
-            self.symbol_table.add(
-                RoutineSymbol(new_name, old_symbol.datatype),
-                tag='own_routine_symbol')
+            self.symbol_table.rename_symbol(symbol, new_name)
 
     def __str__(self):
         result = self.node_str(False) + ":\n"
@@ -234,15 +260,30 @@ class Routine(Schedule, CommentableMixin):
                           symbol table of this Routine.
         '''
         if not isinstance(value, DataSymbol):
-            raise TypeError("Routine return-symbol should be a DataSymbol "
-                            "but found '{0}'".format(
-                                type(value).__name__))
+            raise TypeError(f"Routine return-symbol should be a DataSymbol "
+                            f"but found '{type(value).__name__}'.")
         if value not in self.symbol_table.local_datasymbols:
             raise KeyError(
-                "For a symbol to be a return-symbol, it must be present in "
-                "the symbol table of the Routine but '{0}' is not.".format(
-                    value.name))
+                f"For a symbol to be a return-symbol, it must be present in "
+                f"the symbol table of the Routine but '{value.name}' is not.")
         self._return_symbol = value
+        # The routine symbol must be updated accordingly, this is because the
+        # function datatype is provided by the type of the return symbol which
+        # may be given after the Routine is created.
+        self.symbol_table.lookup(self._name).datatype = value.datatype
+
+    def _refine_copy(self, other):
+        ''' Refine the object attributes when a shallow copy is not the most
+        appropriate operation during a call to the copy() method.
+
+        :param other: object we are copying from.
+        :type other: :py:class:`psyclone.psyir.node.Routine`
+
+        '''
+        super()._refine_copy(other)
+        if other.return_symbol is not None:
+            self.return_symbol = self.symbol_table.lookup(
+                    other.return_symbol.name)
 
 
 # For automatic documentation generation
